@@ -16,30 +16,36 @@
 """Keypair action implementations"""
 
 import io
+import logging
 import os
-import six
 import sys
 
-from openstackclient.common import command
-from openstackclient.common import exceptions
-from openstackclient.common import utils
+from osc_lib.command import command
+from osc_lib import exceptions
+from osc_lib import utils
+import six
+
 from openstackclient.i18n import _
 
 
+LOG = logging.getLogger(__name__)
+
+
 class CreateKeypair(command.ShowOne):
-    """Create new public key"""
+    _description = _("Create new public or private key for server ssh access")
 
     def get_parser(self, prog_name):
         parser = super(CreateKeypair, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<name>',
-            help=_("New public key name")
+            help=_("New public or private key name")
         )
         parser.add_argument(
             '--public-key',
             metavar='<file>',
-            help=_("Filename for public key to add")
+            help=_("Filename for public key to add. If not used, "
+                   "creates a private key.")
         )
         return parser
 
@@ -77,24 +83,41 @@ class CreateKeypair(command.ShowOne):
 
 
 class DeleteKeypair(command.Command):
-    """Delete public key"""
+    _description = _("Delete public or private key(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteKeypair, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<key>',
-            help=_("Public key to delete (name only)")
+            nargs='+',
+            help=_("Name of key(s) to delete (name only)")
         )
         return parser
 
     def take_action(self, parsed_args):
         compute_client = self.app.client_manager.compute
-        compute_client.keypairs.delete(parsed_args.name)
+        result = 0
+        for n in parsed_args.name:
+            try:
+                data = utils.find_resource(
+                    compute_client.keypairs, n)
+                compute_client.keypairs.delete(data.name)
+            except Exception as e:
+                result += 1
+                LOG.error(_("Failed to delete key with name "
+                          "'%(name)s': %(e)s")
+                          % {'name': n, 'e': e})
+
+        if result > 0:
+            total = len(parsed_args.name)
+            msg = (_("%(result)s of %(total)s keys failed "
+                   "to delete.") % {'result': result, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListKeypair(command.Lister):
-    """List public key fingerprints"""
+    _description = _("List key fingerprints")
 
     def take_action(self, parsed_args):
         compute_client = self.app.client_manager.compute
@@ -111,20 +134,20 @@ class ListKeypair(command.Lister):
 
 
 class ShowKeypair(command.ShowOne):
-    """Display public key details"""
+    _description = _("Display key details")
 
     def get_parser(self, prog_name):
         parser = super(ShowKeypair, self).get_parser(prog_name)
         parser.add_argument(
             'name',
             metavar='<key>',
-            help=_("Public key to display (name only)")
+            help=_("Public or private key to display (name only)")
         )
         parser.add_argument(
             '--public-key',
             action='store_true',
             default=False,
-            help=_("Show only bare public key")
+            help=_("Show only bare public key paired with the generated key")
         )
         return parser
 
