@@ -20,6 +20,7 @@ import sys
 
 from keystoneauth1 import exceptions as ks_exc
 from osc_lib.command import command
+from osc_lib import exceptions
 from osc_lib import utils
 import six
 
@@ -109,7 +110,8 @@ def _process_identity_and_resource_options(parsed_args,
 
 
 class AddRole(command.Command):
-    """Adds a role assignment to a user or group on a domain or project"""
+    _description = _("Adds a role assignment to a user or group on a domain "
+                     "or project")
 
     def get_parser(self, prog_name):
         parser = super(AddRole, self).get_parser(prog_name)
@@ -151,7 +153,7 @@ class AddRole(command.Command):
 
 
 class CreateRole(command.ShowOne):
-    """Create new role"""
+    _description = _("Create new role")
 
     def get_parser(self, prog_name):
         parser = super(CreateRole, self).get_parser(prog_name)
@@ -198,7 +200,7 @@ class CreateRole(command.ShowOne):
 
 
 class DeleteRole(command.Command):
-    """Delete role(s)"""
+    _description = _("Delete role(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteRole, self).get_parser(prog_name)
@@ -222,18 +224,30 @@ class DeleteRole(command.Command):
         if parsed_args.domain:
             domain_id = common.find_domain(identity_client,
                                            parsed_args.domain).id
-
+        errors = 0
         for role in parsed_args.roles:
-            role_obj = utils.find_resource(
-                identity_client.roles,
-                role,
-                domain_id=domain_id
-            )
-            identity_client.roles.delete(role_obj.id)
+            try:
+                role_obj = utils.find_resource(
+                    identity_client.roles,
+                    role,
+                    domain_id=domain_id
+                )
+                identity_client.roles.delete(role_obj.id)
+            except Exception as e:
+                errors += 1
+                LOG.error(_("Failed to delete role with "
+                          "name or ID '%(role)s': %(e)s"),
+                          {'role': role, 'e': e})
+
+        if errors > 0:
+            total = len(parsed_args.roles)
+            msg = (_("%(errors)s of %(total)s roles failed "
+                   "to delete.") % {'errors': errors, 'total': total})
+            raise exceptions.CommandError(msg)
 
 
 class ListRole(command.Lister):
-    """List roles"""
+    _description = _("List roles")
 
     def get_parser(self, prog_name):
         parser = super(ListRole, self).get_parser(prog_name)
@@ -371,7 +385,8 @@ class ListRole(command.Lister):
 
 
 class RemoveRole(command.Command):
-    """Removes a role assignment from domain/project : user/group"""
+    _description = _("Removes a role assignment from domain/project : "
+                     "user/group")
 
     def get_parser(self, prog_name):
         parser = super(RemoveRole, self).get_parser(prog_name)
@@ -415,7 +430,7 @@ class RemoveRole(command.Command):
 
 
 class SetRole(command.Command):
-    """Set role properties"""
+    _description = _("Set role properties")
 
     def get_parser(self, prog_name):
         parser = super(SetRole, self).get_parser(prog_name)
@@ -452,7 +467,7 @@ class SetRole(command.Command):
 
 
 class ShowRole(command.ShowOne):
-    """Display role details"""
+    _description = _("Display role details")
 
     def get_parser(self, prog_name):
         parser = super(ShowRole, self).get_parser(prog_name)
