@@ -32,13 +32,15 @@ from openstackclient.identity import common
 
 DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
+DISK_CHOICES = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vhdx",
+                "vdi", "iso", "ploop"]
 
 
 LOG = logging.getLogger(__name__)
 
 
 def _format_image(image):
-    """Format an image to make it more consistent with OSC operations. """
+    """Format an image to make it more consistent with OSC operations."""
 
     info = {}
     properties = {}
@@ -69,7 +71,7 @@ def _format_image(image):
 
 
 class AddProjectToImage(command.ShowOne):
-    """Associate project with image"""
+    _description = _("Associate project with image")
 
     def get_parser(self, prog_name):
         parser = super(AddProjectToImage, self).get_parser(prog_name)
@@ -107,7 +109,7 @@ class AddProjectToImage(command.ShowOne):
 
 
 class CreateImage(command.ShowOne):
-    """Create/upload an image"""
+    _description = _("Create/upload an image")
 
     deadopts = ('size', 'location', 'copy-from', 'checksum', 'store')
 
@@ -140,9 +142,10 @@ class CreateImage(command.ShowOne):
         parser.add_argument(
             "--disk-format",
             default=DEFAULT_DISK_FORMAT,
+            choices=DISK_CHOICES,
             metavar="<disk-format>",
-            help=_("Image disk format "
-                   "(default: %s)") % DEFAULT_DISK_FORMAT,
+            help=_("Image disk format. The supported options are: %s. "
+                   "The default format is: raw") % ', '.join(DISK_CHOICES)
         )
         parser.add_argument(
             "--min-disk",
@@ -361,7 +364,7 @@ class CreateImage(command.ShowOne):
 
 
 class DeleteImage(command.Command):
-    """Delete image(s)"""
+    _description = _("Delete image(s)")
 
     def get_parser(self, prog_name):
         parser = super(DeleteImage, self).get_parser(prog_name)
@@ -398,7 +401,7 @@ class DeleteImage(command.Command):
 
 
 class ListImage(command.Lister):
-    """List available images"""
+    _description = _("List available images")
 
     def get_parser(self, prog_name):
         parser = super(ListImage, self).get_parser(prog_name)
@@ -447,8 +450,9 @@ class ListImage(command.Lister):
         parser.add_argument(
             '--sort',
             metavar="<key>[:<direction>]",
+            default='name:asc',
             help=_("Sort output by selected keys and directions(asc or desc) "
-                   "(default: asc), multiple keys and directions can be "
+                   "(default: name:asc), multiple keys and directions can be "
                    "specified separated by comma"),
         )
         parser.add_argument(
@@ -482,7 +486,6 @@ class ListImage(command.Lister):
         if parsed_args.marker:
             kwargs['marker'] = utils.find_resource(image_client.images,
                                                    parsed_args.marker).id
-
         if parsed_args.long:
             columns = (
                 'ID',
@@ -515,7 +518,19 @@ class ListImage(command.Lister):
             column_headers = columns
 
         # List of image data received
-        data = image_client.api.image_list(**kwargs)
+        data = []
+        if 'marker' in kwargs:
+            data = image_client.api.image_list(**kwargs)
+        else:
+            # No pages received yet, so start the page marker at None.
+            marker = None
+            while True:
+                page = image_client.api.image_list(marker=marker, **kwargs)
+                if not page:
+                    break
+                data.extend(page)
+                # Set the marker to the id of the last item we received
+                marker = page[-1]['id']
 
         if parsed_args.property:
             # NOTE(dtroyer): coerce to a list to subscript it in py3
@@ -542,7 +557,7 @@ class ListImage(command.Lister):
 
 
 class RemoveProjectImage(command.Command):
-    """Disassociate project with image"""
+    _description = _("Disassociate project with image")
 
     def get_parser(self, prog_name):
         parser = super(RemoveProjectImage, self).get_parser(prog_name)
@@ -575,7 +590,7 @@ class RemoveProjectImage(command.Command):
 
 
 class SaveImage(command.Command):
-    """Save an image locally"""
+    _description = _("Save an image locally")
 
     def get_parser(self, prog_name):
         parser = super(SaveImage, self).get_parser(prog_name)
@@ -603,7 +618,7 @@ class SaveImage(command.Command):
 
 
 class SetImage(command.Command):
-    """Set image properties"""
+    _description = _("Set image properties")
 
     deadopts = ('visibility',)
 
@@ -650,8 +665,9 @@ class SetImage(command.Command):
         parser.add_argument(
             "--disk-format",
             metavar="<disk-format>",
-            help=_("Image disk format "
-                   "(default: %s)") % DEFAULT_DISK_FORMAT,
+            choices=DISK_CHOICES,
+            help=_("Image disk format. The supported options are: %s") %
+            ', '.join(DISK_CHOICES)
         )
         protected_group = parser.add_mutually_exclusive_group()
         protected_group.add_argument(
@@ -761,6 +777,23 @@ class SetImage(command.Command):
                 dest=deadopt.replace('-', '_'),
                 help=argparse.SUPPRESS,
             )
+
+        membership_group = parser.add_mutually_exclusive_group()
+        membership_group.add_argument(
+            "--accept",
+            action="store_true",
+            help=_("Accept the image membership"),
+        )
+        membership_group.add_argument(
+            "--reject",
+            action="store_true",
+            help=_("Reject the image membership"),
+        )
+        membership_group.add_argument(
+            "--pending",
+            action="store_true",
+            help=_("Reset the image membership to 'pending'"),
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -812,12 +845,14 @@ class SetImage(command.Command):
             project_arg = parsed_args.owner
             LOG.warning(_('The --owner option is deprecated, '
                           'please use --project instead.'))
+        project_id = None
         if project_arg:
-            kwargs['owner'] = common.find_project(
+            project_id = common.find_project(
                 identity_client,
                 project_arg,
                 parsed_args.project_domain,
             ).id
+            kwargs['owner'] = project_id
 
         image = utils.find_resource(
             image_client.images, parsed_args.image)
@@ -830,6 +865,21 @@ class SetImage(command.Command):
             image_client.images.reactivate(image.id)
             activation_status = "activated"
 
+        membership_group_args = ('accept', 'reject', 'pending')
+        membership_status = [status for status in membership_group_args
+                             if getattr(parsed_args, status)]
+        if membership_status:
+            # If a specific project is not passed, assume we want to update
+            # our own membership
+            if not project_id:
+                project_id = self.app.client_manager.auth_ref.project_id
+            # The mutually exclusive group of the arg parser ensure we have at
+            # most one item in the membership_status list.
+            if membership_status[0] != 'pending':
+                membership_status[0] += 'ed'  # Glance expects the past form
+            image_client.image_members.update(
+                image.id, project_id, membership_status[0])
+
         if parsed_args.tags:
             # Tags should be extended, but duplicates removed
             kwargs['tags'] = list(set(image.tags).union(set(parsed_args.tags)))
@@ -844,7 +894,7 @@ class SetImage(command.Command):
 
 
 class ShowImage(command.ShowOne):
-    """Display image details"""
+    _description = _("Display image details")
 
     def get_parser(self, prog_name):
         parser = super(ShowImage, self).get_parser(prog_name)
@@ -867,7 +917,7 @@ class ShowImage(command.ShowOne):
 
 
 class UnsetImage(command.Command):
-    """Unset image tags and properties"""
+    _description = _("Unset image tags and properties")
 
     def get_parser(self, prog_name):
         parser = super(UnsetImage, self).get_parser(prog_name)
